/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004, 2005

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;


namespace ShopClassLibrary
{
	/// <summary>
	/// Summary description for interface ICustomer.
	/// </summary>
	public interface ICustomer
	{
		DataSet GetById(string user, string pass, long clientId);

		DataSet FindByName(string user, string pass, string namePattern);

		long Add(string user, string pass, 
			     string name, string address, string phone, 
				 string fax, string email, out ShopStatus status);

		ShopStatus Update(string user, string pass, 
			              long customerId, string name, string address, 
				    	  string phone, string fax, string email);
	}

	/// <summary>
	/// Summary description for Customer.
	/// </summary>
	internal class Customer : ICustomer
	{
		public Customer()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="user"></param>
		/// <param name="pass"></param>
		/// <param name="clientId"></param>
		/// <returns></returns>
		public DataSet GetById(string user, string pass, long clientId)
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				ShopStatus status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status != ShopStatus.Ok)
					return null;

				//
				// efectuar pesquisa
				//
				ds = UtilDB.GetById(conn, null, "Customers", "CustomerID", clientId);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetByID> da classe 'Customer': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="user"></param>
		/// <param name="pass"></param>
		/// <param name="namePattern"></param>
		/// <returns></returns>
		public DataSet FindByName(string user, string pass, string namePattern)
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				ShopStatus status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status != ShopStatus.Ok)
					return null;

				//
				// efectuar pesquisa
				//
				ds = UtilDB.FindByStringField(conn, "Customers", "Name", namePattern);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetByID> da classe 'Customer': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="user"></param>
		/// <param name="pass"></param>
		/// <param name="name"></param>
		/// <param name="address"></param>
		/// <param name="phone"></param>
		/// <param name="fax"></param>
		/// <param name="email"></param>
		/// <param name="status"></param>
		/// <returns></returns>
		public long Add(string user, string pass, 
			            string name, string address, string phone, string fax, 
			            string email, out ShopStatus status)
		{
			long customerID = -1;
			OleDbConnection conn = null;

			//
			// validar dados de entrada
			//
			if (name == null || name.Trim().Length == 0)
			{
				status = ShopStatus.InvalidArgument;
				return -1;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status!= ShopStatus.Ok)
					return -1;

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "INSERT INTO Customers (Name, Address, PhoneNb, FaxNb, EMail) Values (?, ?, ?, ?, ?)";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
                cmd.Parameters.AddWithValue("name", name);
				if (address == null || address.Length == 0)
                    cmd.Parameters.AddWithValue("address", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("address", address);
				if (phone == null || phone.Length == 0)
                    cmd.Parameters.AddWithValue("phone", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("phone", phone);
				if (fax == null || fax.Length == 0)
                    cmd.Parameters.AddWithValue("fax", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("fax", fax);
				if (email == null || email.Length == 0)
                    cmd.Parameters.AddWithValue("email", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("email", email);
		
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = ShopStatus.NotOk;
				else
				{
					//
					// inseriu registo
					// obter novo cdigo de id gerado pela BD
					//
					OleDbCommand idCmd = new OleDbCommand("SELECT @@IDENTITY", conn);
					customerID = (int)idCmd.ExecuteScalar();
					status = ShopStatus.Ok;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <AddCustomer> da classe 'Customer': " + ex.Message);
				status = ShopStatus.Error;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return customerID;
		}

		public ShopStatus Update(string user, string pass, 
			                     long customerId, string name, string address, 
								 string phone, string fax, string email)
		{
			OleDbConnection conn = null;
			ShopStatus status;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pass);
				if (status != ShopStatus.Ok)
					return status;

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "Update Customers SET Name = ?, Address = ?, PhoneNb = ?, FaxNb = ?, EMail = ? WHERE CustomerID = ?";
				OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn);
                cmd.Parameters.AddWithValue("name", name);
				if (address.Length == 0)
                    cmd.Parameters.AddWithValue("address", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("address", address);
				if (phone.Length == 0)
                    cmd.Parameters.AddWithValue("phone", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("phone", phone);
				if (fax.Length == 0)
                    cmd.Parameters.AddWithValue("fax", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("fax", fax);
				if (email.Length == 0)
                    cmd.Parameters.AddWithValue("email", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("email", email);
                cmd.Parameters.AddWithValue("custid", customerId);

				//
				// executar o comando 
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = ShopStatus.NotOk;
				else
					// actualizou registo
					status = ShopStatus.Ok;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Update> da classe 'Customer': " + ex.Message);
				status = ShopStatus.Error;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}
	}
}
