/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;

namespace ShopClassLibrary
{
	/// <summary>
	/// Summary description for Comum.
	/// </summary>
	internal class UtilDB
	{
		private const string DEFAULT_CONN = @"Provider=Microsoft.Jet.OLEDB.4.0;User ID=;Password=;Mode=Share Deny None;Data Source=";

		public static string CONN
		{
			get 
			{
				string dbPath =	System.Configuration.ConfigurationSettings.AppSettings["DBPath"];

				if (dbPath == null || dbPath.Length == 0)
				{
					throw new ApplicationException("Falta configurar o caminho para a BD 'Sales'");
				}

				return DEFAULT_CONN + dbPath; 
			}
		}

		private UtilDB()
		{
		}

		#region ValidateUser

		/// <summary>
		/// Valida um login/password no sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns></returns>
		static public ShopStatus ValidateUser(string user, string pass)
		{
			ShopStatus status = ShopStatus.Ok;
			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(CONN);
				conn.Open();

				//
				// invocar mtodo auxiliar
				//
				status = ValidateUser(conn, null, user, pass);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO na validao do 'User': " + ex);
				status = ShopStatus.Error;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Valida um login/password no sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns></returns>
		static public ShopStatus ValidateUser(OleDbConnection conn, OleDbTransaction tx, string user, string pass)
		{
			ShopStatus status = ShopStatus.Ok;
			OleDbCommand    cmd  = null;

			try
			{
				//
				// criar comando SQL a executar
				//
// <--- ALTERAR PARA USAR PARAMETROS : SEGURANA --->
				cmd = new OleDbCommand("Select * from Users Where User='" + user + "' and Pass='" + pass + "'", conn);
				cmd.Transaction = tx;

				//
				// executar o comando e obter um DataReader
				// poderia ser feito usando um count(*) e um ExecuteScalar()
				//
				OleDbDataReader dr = cmd.ExecuteReader();

				//
				// verificar se retornou dados 
				// ( necessrio manter a conexo aberta pois estamos a usar um DataReader)
				//
				if (dr.HasRows)
					status = ShopStatus.Ok;
				else
					status = ShopStatus.InvalidLogOn;

				dr.Close();
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO na validao do 'User': " + ex);
				status = ShopStatus.Error;
			}

			return status;
		}
		#endregion

		#region GetById

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		static public DataSet GetById(string table, string keyName, long key)
		{
			DataSet ds = null;
			OleDbConnection conn = null;

			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(CONN);
				conn.Open();

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				string sSqlCmd = "Select * From "+ table + " Where "+ keyName + " = "+ key.ToString();
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, table);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'UtilDB.GetByID': " + ex.Message);
				ds = null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="conn">a conexo a utilizar</param>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="keyName">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		static public DataSet GetById(OleDbConnection conn, OleDbTransaction tx, string table, string keyName, long key)
		{
			DataSet ds = null;

			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				string sSqlCmd = "Select * From "+ table + " Where "+ keyName + " = "+ key.ToString();
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Transaction = tx;

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, table);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'UtilDB.GetByID': " + ex.Message);
				ds = null;
			}

			return ds;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		static public DataSet GetById(OleDbConnection conn, OleDbTransaction tx, string table, string keyName, int[] keys)
		{
			DataSet ds = null;
			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				System.Text.StringBuilder sb = new System.Text.StringBuilder("Select * From "+ table + " Where "+ keyName + " IN (");
				foreach (int k in keys)
				{
					sb.Append(k.ToString());
					sb.Append(",");
				}
				sb.Length = sb.Length-1;
				sb.Append(")");
				string sSqlCmd = sb.ToString();

				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Transaction = tx;

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, table);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'UtilDB.GetByID': " + ex.Message);
				ds = null;
			}

			return ds;
		}
		
		#endregion

		#region FindByStringField

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		static public DataSet FindByStringField(string table, string field, string key)
		{
			OleDbConnection conn = null;
			DataSet ret = new DataSet();

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(UtilDB.CONN);

				//
				// invocar mtodo auxiliar
				//
				ret = FindByStringField(conn, table, field, key);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State != ConnectionState.Closed)
					conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		static public DataSet FindByStringField(OleDbConnection conn, string table, string field, string key)
		{
			DataSet ret = new DataSet();

			try
			{
				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM " + table + " WHERE " + field + " LIKE ?", conn);
				da.SelectCommand.Parameters.AddWithValue("key", key+"%");

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret, table);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}

			//
			// retornar valor
			//
			return ret;
		}

		#endregion
	}
}
